#!/bin/bash
##########################################################################
# build_gc3_mingw.sh - Build GnuCOBOL 3.x-dev for MinGW in Windoze
# -------------------------------------------------------------------
# This script should work for current and future versions of GnuCobol, 
# with only minor modification to the parameteres and script.
#
# The script MUST be invoked from Windoze by "build_gc3_mingw.bat". Please
# see that script for additional and essential procedural documentation.
# -----------------------------------------------------------------------
# Author: Carl D. Goldin <cdgoldin@gmail.com> - copyright 2019-2020
# Source: Based on Arnold Trembly's GnuCobol Build guide
#         (https://www.arnoldtrembley.com/GnuCOBOL.htm)
#         and very helpful information from Anon, Simon, David, 
#         Ken, and others on the GnuCobol forum:
#         https://sourceforge.net/p/open-cobol/discussion/help/
# Date-written:  12/26/2019
# Last-revision: 01/10/2020
#
# Note: some parts of this script haven't been thoroughly tested yet.
# Please report any bugs and/or suggestions to cdgoldin@gmail.com. Thank you.
#
##########################################################################
# About a dozen environmental variables are set by the Windoze script, and
# and passed to this script via "EXPORT" commands. If you don't see a 
# variable defined below, it SHOULD be defined in the Windoze script. A
# list of exported variables is printed to the screen at program start time.
##########################################################################
msysfolder="/mingw/msys" #do NOT change this variable!

printf "\nBegin MSYS bash script to build $gc3version\n\n" 

##########################################################################
# global functions
##########################################################################
function pause ()
{ read -p "Press any key to continue" pause
  echo "";
}

function press-any-key ()
{ echo "";
  read -p "Press any key to continue - Control C to cancel" pause
  echo "";
}

function check-for-errors ()
{ echo "";
  read -p "Check for errors - Press any key to continue - Control C to cancel" pause
  echo "";
}

function yes_or_no () #messagetext
{ reply="no";
  while [[ "$reply" != "y" && "$reply" != "n" ]]
  do 
    read -p "$1? y/n " reply;
    if [ "$reply" == "Y" ]; then reply = "y"; fi;
    if [ "$reply" == "N" ]; then reply = "n"; fi;
  done
}

another="a";

function grab_a_beer ()
{ printf "grab $another beer";
  another="another";
} 

##########################################################################
# Download and install unzip.exe (needed for unzipping VBISAM...zip)
##########################################################################
uzpversion=unzip-6.0-1

if [ ! -f $msysfolder/bin/unzip.exe ]; then 
   echo " "
   echo "Downloading $upzversion" 
   wget -O $tarfolder/$uzpversion.tar.lzma https://sourceforge.net/projects/mingw/files/MSYS/Extension/unzip/$uzpversion/$uzpversion-msys-1.0.13-bin.tar.lzma/download
   if [ $? -gt 0 ]; then 
       echo " "
       echo "Download of $uzpversion.tar.lzma failed - correct problem and restart\n"
       pause; exit 8
   fi

   cd $msysfolder 

   echo " "
   echo "Unpacking $uzpversion"
   tar --lzma -xpf $tarfolder/$uzpversion.tar.lzma bin/unzip.exe
 
   if [ $? -gt 0 ]; then
      echo " "
      echo "Unpack of $uzpversion.tar.lzma failed - correct problem and restart\n"
      pause; exit 8
   fi
       
   rm -i $tarfolder/$uzpversion.tar.lzma
fi

##########################################################################
# Install GMP (GNU Multiple-Precision Arithmetic Library) 
##########################################################################
if [ "$MATHLIB" == "GMP" ]; then
   yes_or_no "Install GMP (GNU Multiple-Precision Arithmetic Library)";
else
   reply="n";   
fi   

if [ "$reply" == "y" ]; then
   echo "Unpacking $gmpversion tarball - this should take 1-2 minutes"
   cd $msysfolder      
   tar xf $tarfolder/$gmpversion.tar.xz  
   if [ $? -gt 0 ]; then
      echo "Unpack of $tarfolder/$gmpversion.tar.xz failed - correct problem and restart\n"
      pause; exit 8; fi
      
   rm -i $tarfolder/$gmpversion.tar.xz     

   if [ ! -d $msysfolder/$gmpversion ]; then
      echo "$msysfolder/$gmpversion does not exist - cancelling"
      pause; exit 8
   fi

   echo " "
   echo "Installing GMP"
   echo "./configure will take about 5 minutes and produce hundreds of messages"
   echo "'make' will take about 10 minutes and produce thousands of messages"
   echo "'make check' will take about 15 minutes and produce thousands of messages"
   printf "'make install' runs very quickly --- so " grab_a_beer
   printf " and wait.\n"
   press-any-key
   
   cd $msysfolder/$gmpversion           
   ./configure --prefix=/mingw --disable-static --enable-shared  
   if [ $? -gt 0 ]; then check-for-errors; fi  
   make
   if [ $? -gt 0 ]; then check-for-errors; fi
   make check
   if [ $? -gt 0 ]; then check-for-errors; fi
   make install
   if [ $? -gt 0 ]; then check-for-errors; fi

   ls /mingw/bin/libgmp*.dll > /dev/null
   if [ $? -gt 0 ]; then 
      echo "/mingw/bin/libgmp*.dll was not created - cancelling" 
      pause; exit 8
   fi
   
   echo " "
fi

##########################################################################
# Install MPIR (Multiple Precision Integers and Rationals)
##########################################################################
if [ "$MATHLIB" == "MPIR" ]; then
   yes_or_no "Install MPIR (Multiple Precision Integers and Rationals)"
else
   reply="n";   
fi   
    
if [ "$reply" == "y" ]; then
   printf "Unpacking tarball - this may take a while - " grab_a_beer
   printf ".\n"
   cd $msysfolder
   tar xjf $tarfolder/$mpirversion.tar.bz2
   if [ $? -gt 0 ]; then
      echo "Unpack of $tarfolder/$mpirversion.tar.bz2 failed  - correct problem and restart\n"
      pause; exit 8; fi 
   
   rm -i $tarfolder/$mpirversion.tar.bz2
 
   if [ ! -d $msysfolder/$mpirversion ]; then
      echo "$msysfolder/$mpirversion does not exist - cancelling"
      pause; exit 8
   fi    

   echo " "
   echo "Installing MPIR"
   echo "./configure will take about 5 minutes and produce hundreds of messages"
   echo "'make' will take about 10 minutes and produce thousands of messages"
   echo "'make check' will take about 15 minutes and produce thousands of messages"
   printf "'make install' runs very quickly --- so " grab_a_beer
   printf ".\n"
   press-any-key
 
   cd $msysfolder/$mpirversion      
   ./configure --prefix=/mingw --disable-static --enable-shared --enable-gmpcompat --with-yasm=$msysfolder/$yasmversion
   if [ $? -gt 0 ]; then check-for-errors; fi  
   make                                      
   if [ $? -gt 0 ]; then check-for-errors; fi      
   make check
   if [ $? -gt 0 ]; then check-for-errors; fi
   make install
   if [ $? -gt 0 ]; then check-for-errors; fi

   ls /mingw/bin/libmpir*.dll > /dev/null
   if [ $? -gt 0 ]; then 
      echo "/mingw/bin/libmpir*.dll was not created - cancelling" 
      pause; exit 8
   fi
   
   echo " "
fi

##########################################################################
# Install PDCurses
##########################################################################
yes_or_no "Install $pdcurses"
 
if [ "$reply" == "y" ]; then
   echo "Unpacking tarball - this should only take a few seconds"
   cd $msysfolder       
   tar xzf $tarfolder/$pdcurses.tar.gz  
   if [ $? -gt 0 ]; then
      echo "Unpack of $tarfolder/$pdcurses.tar.gz failed  - correct problem and restart\n"
      pause; exit 8; fi
   
   rm -i $tarfolder/$pdcurses.tar.gz       
 
   echo " "
   echo "Building PDCurses"
   echo "'make' will take a few more seconds"
   echo "'install' runs very quickly --- so no time for a beer."
   press-any-key

   cd $msysfolder

   subdir="?"

   if [ $pdcurses == "PDCurses-3.4" ];   then subdir="win32";  fi
   if [ $pdcurses == "PDCurses-3.9" ];   then subdir="wincon"; fi
   if [ $pdcurses == "PDCurses-4.1.0" ]; then
      if [ $wingui == "no" ];            then subdir="wincon"; fi
      if [ $wingui == "yes" ];           then subdir="wingui"; fi
   fi   

   cd $pdcurses/$subdir

   makeparms="?"

   if [ $pdcurses == "PDCurses-3.4" ]; then makeparms="gccwin32.mak DLL=Y"; fi
   if [ $pdcurses == "PDCurses-3.9" ]; then makeparms="Makefile INFOEX=N DLL=Y"; fi
   if [ $pdcurses == "PDCurses-4.1.0" ]; then
      if [ $wingui == "no" ];  then makeparms="Makefile.mng INFOEX=N DLL=Y"; fi
      if [ $wingui == "yes" ]; then 
         if $wide == "yes" ];  then makeparms="Makefile.mng INFOEX=N CHTYPE_32=Y DLL=Y WIDE=Y";
                               else makeparms="Makefile.mng INDOEX=N DLL=Y UTF8=Y";
         fi
      fi
   fi    
   
   make -f $makeparms
   if [ $? -gt 0 ]; then check-for-errors; fi
   # 3.9 make returns: make: Nothing to be done for `all'. ???

   ls pdcurses.dll > /dev/null
   if [ $? -gt 0 ]; then 
      echo "pdcurses.dll was not created - cancelling"  
      pause; exit 8; fi

   install pdcurses.dll /mingw/bin/.
   if [ $? -gt 0 ]; then check-for-errors; fi
   install pdcurses.a /mingw/lib/libpdcurses.a
   if [ $? -gt 0 ]; then check-for-errors; fi
   cd ..
   install *.h /mingw/include/.              
   if [ $? -gt 0 ]; then check-for-errors; fi    
   install curses.h /mingw/include/pdcurses.h
   if [ $? -gt 0 ]; then check-for-errors; fi 

   echo " "
fi   

##########################################################################
# Install Berkely Database (BDB) 
##########################################################################
if [ "$ISAM" == "BDB" ]; then
   yes_or_no "Install Berkely Database (BDB)"
else
   reply="n";   
fi   
    
if [ "$reply" == "y" ]; then
   echo "Unpacking tarball - this should take a few minutes"
   cd $msysfolder       
   tar xzf $tarfolder/$BDBversion.tar.gz   
   if [ $? -gt 0 ]; then
      echo "Unpacking of $tarfolder/$BDBversion.tar.gz failed  - correct problem and restart\n"
      pause; exit 8; fi
   
   rm -i $tarfolder/$BDBversion.tar.gz        
   
   printf "\nEdit os_stat.c, replacing '_tcsclen' with 'strlen'\n"
   sed -i 's/_tcsclen/strlen/g' $msysfolder/$BDBversion/src/os_windows/os_stat.c 
   printf "\ndouble-check that the change was made\n"
   grep "strlen" $msysfolder/$BDBversion/src/os_windows/os_stat.c
   pause 

   printf "\nInstalling BDB\n"
   echo "The 'configure' step runs fairly quickly and produces hundreds of messages."
   echo "The 'make' step runs for a fairly long time."
   echo "The 'make install' step runs very quickly."
   echo "If the BDB build appears to hang at "Installing documentation", let it run"
   echo "You should have time for a beer or two"
   press-any-key
   
   cd $msysfolder/$BDBversion/build_unix               
   ../dist/configure --enable-mingw --prefix=/mingw --enable-compat185 LIBCSO_LIBS=-lwsock32
   if [ $? -gt 0 ]; then check-for-errors; fi 
   make                                      
   if [ $? -gt 0 ]; then check-for-errors; fi            
   make install      
   if [ $? -gt 0 ]; then check-for-errors; fi
 
   if [ ! -f $MinGWfolder/bin/libdb-6.2.dll ]; then 
      echo $MinGWfolder/bin/libdb-6.2.dll was not created - cancelling
      pause; exit 8; fi
    
   echo " "
fi

##########################################################################
# Install VBISAM database (VBI)
##########################################################################
if [ "$ISAM" == "VBISAM" ]; then
   yes_or_no "Install VBISAM database (VBI)"
else
   reply="n";   
fi

if [ "$reply" == "y" ]; then
  echo "Unpacking tarball - this should take a few minutes"
  cd $msysfolder       
  bin/unzip.exe -q -o $tarfolder/$VBIversion.zip
  if [ $? -gt 0 ]; then
      printf "\nUnzip of $tarfolder/$VBIversion.zip failed - correct problem and restart\n"
      pause; exit 8; fi
  
   rm -i $tarfolder/$VBIversion.zip        

   printf "\nInstalling VBISAM\n"
   echo "The 'configure' step runs fairly quickly and produces about 100 messages"
   echo "The 'make' step runs longer and produces more messages."
   echo "The 'make check' step runs very quickly."
   echo "The 'make install' step should also run very quickly."
   echo "So no time for a beer :-("
   press-any-key
   
   cd $msysfolder/$VBIversion             
  ./configure --prefix=/mingw     
   if [ $? -gt 0 ]; then check-for-errors; fi
   make                                     
   if [ $? -gt 0 ]; then check-for-errors; fi             
   make check      
   if [ $? -gt 0 ]; then check-for-errors; fi
   make install      
   if [ $? -gt 0 ]; then check-for-errors; fi

   if [ ! -f $MinGWfolder/bin/libvbisam-1.dll ]; then
      printf "\n$MinGWfolder/bin/libvbisam-1.dll was not created - cancelling\n"
      pause; exit 8; fi
    
   echo " "
fi

##########################################################################
# Install the $gc3version compiler
##########################################################################
yes_or_no "Install the $gc3version compiler"

if [ "$reply" == "y" ]; then
   printf "\nUnpacking $gc3version - this should only take a few minutes\n"

   cd $msysfolder       
   tar xf $tarfolder/$gc3version.tar.gz
   if [ $? -gt 0 ]; then
      echo "Unpacking of $tarfolder/$gc3version.tar.gz failed  - correct problem and restart\n"
      pause; exit 8; fi 
   
   rm -i $tarfolder/$gc3version.tar.gz
   
   grep "IDENTIFICATION DIVISION." $msysfolder/$gc3version/tests/cobol85/newcob.val > /dev/null
   if [ $? -eq 1 ]; then rm        $msysfolder/$gc3version/tests/cobol85/newcob.val; fi
   #if newcob.val is corrupt, remove it and let "make test" download and unpack it
      
   cJSON="yes" #always set cJSON to yes, to avoid configure error
               #since we don't know how to specify "--without-cjson"
   XML2="no"   #always set XML2 to no, to avoid configure error!
               #since we don't know how to install XML2 under MSYS
   
   if [ "$cJSON" == "yes" ]; then
      echo " "
      echo "Downloading cJSON.c and cJSON.h"
      wget -O $msysfolder/$gc3version/libcob/cJSON.c https://raw.githubusercontent.com/DaveGamble/cJSON/master/cJSON.c 
      if [ $? -gt 0 ]; then
         rm $msysfolder/$gc3version/libcob/cJSON.c #because wget creates an empty file 
         printf "\nDownload of $cJson.c failed - correct problem and restart\n"
         pause; exit 8
      fi
      wget -O $msysfolder/$gc3version/libcob/cJSON.h https://raw.githubusercontent.com/DaveGamble/cJSON/master/cJSON.h 
      if [ $? -gt 0 ]; then
         rm $msysfolder/$gc3version/libcob/cJSON.h #because wget creates an empty file 
         printf "\nDownload of $cJson.h failed - correct problem and restart\n"
         pause; exit 8
      fi
   fi        
        
   echo " "
   echo "Installing $gc3version"
   echo "The 'configure' step runs for less than one minute, producing 1-2 screens of messages."
   echo "The 'make' step runs for about 2 minutes, generating hundreds of messages."
   echo "The 'make check' step perform 1015 basic tests, and takes 10-15 minutes to run"
   echo "The 'make test' step generates hundreds of messages, and runs for 5-10 minutes"
   echo "The 'make install' step should run in a minute or less"
   echo "You should have time for a six pack"
   press-any-key
   
   cd $msysfolder/$gc3version             
                                    options="--without-db"
   if [ "$ISAM" == "BDB" ];    then options="--with-db"; fi
   if [ "$ISAM" == "VBISAM" ]; then options="--with-vbisam"; fi
   
   #if [ "$cJSON" == "no" ]; then options="$options --without-cJSON"; fi
   #unfortunately, there is no such option, but what is the correct one?
   
   if [ "$XML2" == "no" ]; then options="$options --without-xml2"; fi
      
   ./configure --prefix=/mingw $options --disable-rpath
   if [ $? -gt 0 ]; then check-for-errors; fi 
   make                                      
   if [ $? -gt 0 ]; then check-for-errors; fi            
   make check      
   if [ $? -gt 0 ]; then check-for-errors; fi
   make test                       
   if [ $? -gt 0 ]; then check-for-errors; fi
   make install
   if [ $? -gt 0 ]; then check-for-errors; fi
   
   echo " "
fi

##########################################################################
# Just when you thought it would never end ...
##########################################################################
printf "Please close the MSYS window, and return to the Windoze script "
printf "to complete the installation\n"

exit
